@@ -15,6 +15,7 @@
 #include <gtk/gtkwindow.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
+#include <libhal.h>
 
 #include "gvm.h"
 
@@ -317,6 +318,39 @@ set_icon (GtkImage *image, GnomeIconTheme *theme, const char *name)
 	}
 }
 
+/** Check if HAL is running
+ *
+ * @return			TRUE if HAL is running and working.
+ *				FALSE otherwise.
+ */
+static gboolean
+check_if_hal_is_running (void)
+{
+	LibHalContext *ctx;
+	char **devices;
+	int nr;
+
+	ctx = hal_initialize (NULL, FALSE);
+	if (!ctx)
+		return FALSE;
+
+	/*
+	 * Do something to ping the HAL daemon - the above functions will
+	 * succeed even if hald is not running, so long as DBUS is.  But we
+	 * want to exit silently if hald is not running, to behave on
+	 * pre-2.6 systems.
+	 */
+	devices = hal_get_all_devices (ctx, &nr);
+	if (!devices) {
+		hal_shutdown (ctx);
+		return FALSE;
+	}
+	hal_free_string_array (devices);
+
+	hal_shutdown (ctx);
+	return TRUE;
+}
+
 #define GLADE_XML_FILE	"gnome-volume-properties.glade"
 
 /*
@@ -331,6 +365,24 @@ init_capplet (void)
 	gchar *glade_file;
 	char *file;
 
+	/* bail out now if hald is not even running */
+	if (!check_if_hal_is_running ()) {
+		GtkWidget *dialog;
+
+		dialog = gtk_message_dialog_new (NULL,
+						 0, GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_OK,
+						 _("Volume management not supported"));
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+							  _("The \"hald\" service is required but not currently "
+							    "running. Enable the service and rerun this applet, "
+							    "or contact your system administrator."));
+
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+		exit (EXIT_FAILURE);
+	}
+
 	config.client = gconf_client_get_default ();
 	gconf_client_add_dir (config.client, GCONF_ROOT_SANS_SLASH,
 			      GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
@@ -345,17 +397,13 @@ init_capplet (void)
 		dialog = gtk_message_dialog_new (NULL,
 						 0, GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_OK,
-						 "<b>%s</b>\n%s.",
 						 _("Could not load the main "
-						   "interface"),
-						 _("Please make sure that the "
-						   "volume manager is properly "
-						   "installed"));
-		gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-		gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-		gtk_label_set_use_markup (GTK_LABEL
-					  (GTK_MESSAGE_DIALOG (dialog)->label),
-					  TRUE);
+						   "interface"));
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+				 _("Please make sure that the "
+				   "volume manager is properly "
+				   "installed"));
+
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
 						 GTK_RESPONSE_OK);
 		gtk_dialog_run (GTK_DIALOG (dialog));
@@ -368,22 +416,17 @@ init_capplet (void)
 	icon_theme = gnome_icon_theme_new ();
 
 	icon = glade_xml_get_widget (xml, "removable_drives_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-zipdisk");
+	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-removable");
 	icon = glade_xml_get_widget (xml, "audio_cd_image");
 	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-cdrom-audio");
 	icon = glade_xml_get_widget (xml, "blank_cd_image");
-	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-cdrom");
+	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-disc-cdr");
 	icon = glade_xml_get_widget (xml, "dvd_video_image");
 	set_icon (GTK_IMAGE (icon), icon_theme, "gnome-dev-dvd");
 	icon = glade_xml_get_widget (xml, "digital_camera_image");
 	set_icon (GTK_IMAGE (icon), icon_theme, "applets-screenshooter");
 
-	file = gnome_icon_theme_lookup_icon (icon_theme,
-			"gnome-dev-cdrom", 48, NULL, NULL);
-	if (file) {
-		gtk_window_set_default_icon_from_file (file, NULL);
-		g_free (file);
-	}
+	gtk_window_set_default_icon_name ("gnome-dev-cdrom");
 
 	g_object_unref (icon_theme);
 
